package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.rule.RuleClassifyDTO;
import com.gic.enterprise.dto.rule.RuleClassifySceneDTO;
import com.gic.enterprise.dto.rule.RuleDTO;
import com.gic.enterprise.entity.TabConfigRule;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.ConfigRuleApiService;
import com.gic.enterprise.service.ConfigRuleService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("configRuleApiService")
public class ConfigRuleApiServiceImpl implements ConfigRuleApiService {
    @Autowired
    private ConfigRuleService configRuleService;

    @Override
    public ServiceResponse<Integer> saveRule(RuleDTO dto) {
        if (StringUtils.isBlank(dto.getName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则名称不能为空");
        }
        if (StringUtils.isBlank(dto.getAppName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用名称不能为空");
        }
        if (StringUtils.isBlank(dto.getAppCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用code不能为空");
        }
        //名称唯一性校验
        if (configRuleService.isRepeatName(null, 1, dto.getName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则名称重复");
        }
        //填充
        dto.setParentRuleId(0);
        dto.setRuleLevel(1);
        Integer ruleId = configRuleService.saveRule(dto);
        //更新操作ID链数据
        configRuleService.update(new TabConfigRule().setRuleId(ruleId).setRuleChain("_0_" + ruleId + "_"));

        return ServiceResponse.success(ruleId);
    }

    @Override
    public ServiceResponse<Integer> saveRuleClassify(RuleClassifyDTO dto) {
        TabConfigRule record = configRuleService.getByRuleId(dto.getParentRuleId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，规则不存在");
        }
        if (record.getRuleLevel() != 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，不是规则类型");
        }
        if (StringUtils.isBlank(dto.getName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类名称不能为空");
        }
        if (StringUtils.isBlank(dto.getClassifyIcon())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类icon不能为空");
        }
        if (StringUtils.isBlank(dto.getDesc())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类描述不能为空");
        }
        dto.setRuleLevel(2);

        Integer ruleId = configRuleService.saveRuleClassify(dto);
        //更新操作ID链数据
        configRuleService.update(
                new TabConfigRule().setRuleId(ruleId).setRuleChain("_0_" + dto.getParentRuleId() + "_" + ruleId + "_"));

        return ServiceResponse.success(ruleId);
    }

    @Override
    public ServiceResponse<Integer> saveRuleClassifyScene(RuleClassifySceneDTO dto) {
        TabConfigRule record = configRuleService.getByRuleId(dto.getParentRuleId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，规则不存在");
        }
        if (record.getRuleLevel() != 2) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "分类ID错误，不是分类类型");
        }
        if (StringUtils.isBlank(dto.getMarketScene())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景code不能为空");
        }
        if (StringUtils.isBlank(dto.getName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景名称不能为空");
        }
        if (StringUtils.isBlank(dto.getDesc())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "场景描述不能为空");
        }
        Integer marketObject = dto.getMarketObject();
        boolean isNotRightMarketObject = marketObject == null
                || (marketObject != 1 && marketObject != 2 && marketObject != 3);
        if (isNotRightMarketObject) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "营销对象值非法");
        }

        dto.setRuleLevel(3);
        Integer firstRuleId = record.getParentRuleId();
        Integer ruleId = configRuleService.saveRuleClassifyScene(dto);

        //更新操作ID链数据
        configRuleService.update(new TabConfigRule().setRuleId(ruleId)
                .setRuleChain("_0_" + firstRuleId + "_" + dto.getParentRuleId() + "_" + ruleId + "_"));

        return ServiceResponse.success(ruleId);
    }
}
